searchNodes=[{"doc":"A plug building toolkit for blocking and throttling abusive requests. PlugAttack is a set of macros that can be used to build a plug to protect your web app from bad clients. It allows safelisting, blocklisting and throttling based on arbitrary properties of the request. The throttling state is stored in a configurable storage. By default an implementation backed by :ets tables is offered. Example defmodule MyApp.PlugAttack do import Plug.Conn use PlugAttack # For more rules examples see PlugAttack.rule/2 macro documentation. rule &quot;allow local&quot; , conn do allow conn . remote_ip == { 127 , 0 , 0 , 1 } end # It&#39;s possible to customize what happens when conn is let through def allow_action ( conn , _data , _opts ) , do : conn # Or when it&#39;s blocked def block_action ( conn , _data , _opts ) do conn |&gt; send_resp ( :forbidden , &quot;Forbidden \\n &quot; ) |&gt; halt end end","ref":"PlugAttack.html","title":"PlugAttack","type":"behaviour"},{"doc":"Action performed when the request is allowed.","ref":"PlugAttack.html#c:allow_action/3","title":"PlugAttack.allow_action/3","type":"callback"},{"doc":"Action performed when the request is blocked.","ref":"PlugAttack.html#c:block_action/3","title":"PlugAttack.block_action/3","type":"callback"},{"doc":"Defines a rule. A rule is an expression that returns either {:allow, data} , {:block, data} , or nil . If an allow or block tuple is returned we say the rule matched , otherwise the rule didn't match and further rules will be evaluated. If a rule matched the corresponding allow_action/3 or block_action/3 function on the defining module will be called passing the conn , the data value from the allow or block tuple and opts as returned by the init/1 plug callback. If none rule matched, neither allow_action/3 nor block_action/3 will be called. Both actions should behave similarly to plugs, returning the modified conn argument. The default implementation of allow_action/3 will return the conn unmodified. The default implementation of block_action/3 will respond with status 403 Forbidden, the body &quot;Forbidden\\n&quot; and halt the plug pipeline. Various predefined rules are defined in the PlugAttack.Rule module. This module is automatically imported in the rule's body. Examples rule &quot;allow local&quot; , conn do allow conn . remote_ip == { 127 , 0 , 0 , 1 } end rule &quot;block 1.2.3.4&quot; , conn do block conn . remote_ip == { 1 , 2 , 3 , 4 } end rule &quot;throttle per ip&quot; , conn do # throttle to 5 requests per second throttle conn . remote_ip , period : 1_000 , limit : 5 , storage : { PlugAttack.Storage.Ets , MyApp.PlugAttack.Storage } end rule &quot;throttle login requests&quot; , conn do if conn . method == &quot;POST&quot; and conn . path_info == [ &quot;login&quot; ] do throttle conn . params [ &quot;email&quot; ] , period : 60_000 , limit : 10 , storage : { PlugAttack.Storage.Ets , MyApp.PlugAttack.Storage } end end","ref":"PlugAttack.html#rule/3","title":"PlugAttack.rule/3","type":"macro"},{"doc":"The rule return value.","ref":"PlugAttack.html#t:rule/0","title":"PlugAttack.rule/0","type":"type"},{"doc":"Defines various rules that can be used inside the PlugAttack.rule/2 macro.","ref":"PlugAttack.Rule.html","title":"PlugAttack.Rule","type":"module"},{"doc":"The simplest rule that always allows the request to pass. If value is truthy the request is allowed, otherwise next rules are evaluated.","ref":"PlugAttack.Rule.html#allow/1","title":"PlugAttack.Rule.allow/1","type":"function"},{"doc":"The simplest rule that always blocks the request. If value is truthy the request is blocked, otherwise next rules are evaluated.","ref":"PlugAttack.Rule.html#block/1","title":"PlugAttack.Rule.block/1","type":"function"},{"doc":"Implements an algorithm inspired by fail2ban. This intends to catch misbehaving clients early and for longer amounts of time. The key differentiates different clients, you can use, for example, conn.remote_ip for per IP tracking. If the key is falsey the action is skipped and next rules are evaluated. Be careful not to use the same key for different rules that use the same storage. Passes {:fail2ban, key} , as the data to block_action calls when an abusive request is detected. Each misbehaving client is blocked after each call and tracked for :period time. If more than :limit abusive requests are detected within the :period , the client is banned for :ban_for . Options :storage - required, a tuple of PlugAttack.Storage implementation and storage options. :period - required, how long to store abusive requests for counting towards :limit exhaustion. :limit - required, max abusive requests allowed before the ban. :ban_for - required, length of the ban in milliseconds.","ref":"PlugAttack.Rule.html#fail2ban/2","title":"PlugAttack.Rule.fail2ban/2","type":"function"},{"doc":"Implements a request throttling algorithm. The key differentiates different throttles, you can use, for example, conn.remote_ip for per IP throttling, or an email address for login attempts limitation. If the key is falsey the throttling is not performed and next rules are evaluated. Be careful not to use the same key for different rules that use the same storage. Passes {:throttle, data} , as the data to both allow and block tuples, where data is a keyword containing: :period , :limit , :expires_at - when the current limit will expire as unix time in milliseconds, and :remaining - the remaining limit. This can be useful for adding &quot;X-RateLimit-*&quot; headers. Options :storage - required, a tuple of PlugAttack.Storage implementation and storage options. :limit - required, how many requests in a period are allowed. :period - required, how long, in ms, is the period.","ref":"PlugAttack.Rule.html#throttle/2","title":"PlugAttack.Rule.throttle/2","type":"function"},{"doc":"Behaviour for the storage backend for various rules.","ref":"PlugAttack.Storage.html","title":"PlugAttack.Storage","type":"behaviour"},{"doc":"","ref":"PlugAttack.Storage.html#c:increment/4","title":"PlugAttack.Storage.increment/4","type":"callback"},{"doc":"","ref":"PlugAttack.Storage.html#c:read/3","title":"PlugAttack.Storage.read/3","type":"callback"},{"doc":"","ref":"PlugAttack.Storage.html#c:read_sliding_counter/3","title":"PlugAttack.Storage.read_sliding_counter/3","type":"callback"},{"doc":"","ref":"PlugAttack.Storage.html#c:write/4","title":"PlugAttack.Storage.write/4","type":"callback"},{"doc":"","ref":"PlugAttack.Storage.html#c:write_sliding_counter/4","title":"PlugAttack.Storage.write_sliding_counter/4","type":"callback"},{"doc":"","ref":"PlugAttack.Storage.html#t:key/0","title":"PlugAttack.Storage.key/0","type":"type"},{"doc":"","ref":"PlugAttack.Storage.html#t:opts/0","title":"PlugAttack.Storage.opts/0","type":"type"},{"doc":"Time of milliseconds since unix epoch.","ref":"PlugAttack.Storage.html#t:time/0","title":"PlugAttack.Storage.time/0","type":"type"},{"doc":"Storage solution for PlugAttack using a local ets table. Usage You need to start the process in your supervision tree, for example: children = [ # ... worker ( PlugAttack.Storage.Ets , [ MyApp.PlugAttackStorage ] ) ] This will later allow you to pass the :storage option to various rules as storage: {PlugAttack.Storage.Ets, MyApp.PlugAttackStorage}","ref":"PlugAttack.Storage.Ets.html","title":"PlugAttack.Storage.Ets","type":"module"},{"doc":"Forcefully clean the storage.","ref":"PlugAttack.Storage.Ets.html#clean/1","title":"PlugAttack.Storage.Ets.clean/1","type":"function"},{"doc":"Starts the storage table and cleaner process. The process is registered under name and a public, named ets table with that name is created as well. Options :clean_period - how often the ets table should be cleaned of stale data. The key scheme guarantees stale data won't be used for making decisions. This is only about limiting memory consumption (default: 5000 ms).","ref":"PlugAttack.Storage.Ets.html#start_link/2","title":"PlugAttack.Storage.Ets.start_link/2","type":"function"}]